;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe0913ba0fd418cb50258cdc822753b44 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xdd653a2b095791dc430b83ae94c94b13 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x993c6d0bf510114d9dce6ea544353fae . "./code.cg2js/cg-toolbar.fasl")
(#xc53de23b8cf6f73c516deb6ac8f6f43c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xefa37f61ff2f72880e4b4bb84d44588a . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x05fff86c0a7ffcb12390abc0e62969e9 . "./code.cg2js/cg-class-grid.fasl")
(#xff709777ccdd5dcef9824a5945b7b34f . "./code.cg2js/cg-tray.fasl")
(#xf9ce1be2f068cadbf83c6141f4a77a77 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7aa77158342ff39f6c6498f2efff4327 . "./code.cg2js/cg-class-support.fasl")
(#xfcd5e0096dd6f5c14735e378b789578b . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x3994f989cbaaefd16ba882d342ee9baf . "./code.cg2js/cg-dropping-outline.fasl")
(#xf73532302e27077669de7491c8fd3e94 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x039294782bd6e2dead6a9cc6d5c6650c . "./code.cg2js/cg-choice-list.fasl")
(#xd917c1dae3a269de6d058566dab1ca0b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xa145d15cdde597014f991953c100568e . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x05f1c05fa18854e156e817452fb176fa . "./code.cg2js/cg-timer.fasl")
(#x119b7c8bb3efb1d47fda43f0469819eb . "./code.cg2js/cg-directory-dialog.fasl")
(#x9214f2e6cc4e42757114cc1a7b1f1f48 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xef5990cf96224c14195e2995f81f4c63 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x5a4dbcc4ee1ebaffaac01ae036669e99 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x37cb73ac5e0b9aa8fd39bba28bfd4b48 . "./code.cg2js/cg-html-widget.fasl")
(#xc27d7bf33f70d5026a05611995b9c94c . "./code.cg2js/cg-selected-object.fasl")
(#x396728516cd7059e780471d4f8167832 . "./code.cg2js/cg-lamp.fasl")
(#x0e3f804b4fe792719594fd75c3f6b69d . "./code.cg2js/cg-scroll-bar.fasl")
(#x6d4cb0988345b2436cb036d12fa08687 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd1588ec44997de1a7f4c8c1e776aa5fd . "./code.cg2js/cg-message-dialog.fasl")
(#xe7efc4a00c535579f5e4939e0b340ed6 . "./code.cg2js/cg-dde-utils.fasl")
(#x9600b062f71018f8e1a52e44e975c3fd . "./code.cg2js/cg-group-box.fasl")
(#x31697c44f37bbec22e4dafea0390f09d . "./code.cg2js/cg-find-string-dialog.fasl")
(#xabfb1be3d493542460aa702823d043e7 . "./code.cg2js/cg-split-bar.fasl")
(#xc9580d1671f5e82389424fe0263f22f4 . "./code.cg2js/cg-radio-button.fasl")
(#x9c59fe5e80b832568056e93a6e275c50 . "./code.cg2js/cg-base.fasl")
(#x2abdf84a8adfc2f9e83643f52ab8ccad . "./code.cg2js/cg-paren-matching.fasl")
(#xa0d5757f2860372773a326b10bf825fa . "./code.cg2js/cg-graphics-context.fasl")
(#xa4c61729d86407d3cff83f83bf79c201 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xc6af85a1f057f744d65417ed890ebb72 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x0c4e45aba2b526fa1eaa97b7fed60404 . "./code.cg2js/cg-plot-widget.fasl")
(#x37e1e2f837d6a51b8446c0a02fdf0e4f . "./code.cg2js/cg-dev.fasl")
(#x4ef1c13800bb0db69bad2f6b7ed7d08f . "./code.cg2js/cg-text-edit-pane.fasl")
(#xe32af71556149719ae7565042106dd3c . "./code.cg2js/cg-tab-control.fasl")
(#x819330fd2cd5d31d4f53c2399397f2a2 . "./code.cg2js/cg-fill-texture.fasl")
(#x0c475543bbe470fa161db69ba58b1dd9 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xc40f670e099b616b6844f1d3540f7010 . "./code.cg2js/cg-trackbar.fasl")
(#x0457503c1daebe5979e5590e45661e4d . "./code.cg2js/cg-nodes.fasl")
(#x427766360fc5dece0a2b7ac77e829f40 . "./code.cg2js/cg-common-control.fasl")
(#x5bda284a61e7d09621d28587d3e38b88 . "./code.cg2js/cg-list-view.fasl")
(#x0b5125ac53d3778148f65c7c80634f58 . "./code.cg2js/cg-rich-edit.fasl")
(#xe64aa5ed02bfd0cabd9c3e93266c3383 . "./code.cg2js/cg-project-window.fasl")
(#x24ac079cba5e03405b94cec853a35d13 . "./code.cg2js/cg-item-list.fasl")
(#x85dbf73605f17ad12452773225c59194 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4fc6777bf13218bd184fbadee30ff0f1 . "./code.cg2js/cg-get-position.fasl")
(#xd94eab974ac73740c67c1a3061659fcf . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa14a4501e13282cf4cc65a3121dde08b . "./code.cg2js/cg-drawable.fasl")
(#x0b36cea54748e94a914f4c7dc5ed31fe . "./code.cg2js/cg-lettered-menu.fasl")
(#x8b4b17ded55a3c4df4d42ac8263db38b . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xae5fb53e50a93ceb4a5e1a1f658dbd88 . "./code.cg2js/cg-html-dialog.fasl")
(#x11ee4cfc925980eaae5d3af88962fb7f . "./code.cg2js/cg-object-editor-layout.fasl")
(#x9e88f99862ec5968d193996bfed11dfe . "./code.cg2js/cg-utility-dialog.fasl")
(#x913be2f667073bfd3e8511a909da4317 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa2e33dd5b9fb2a3c01f0f6b92e9bfdab . "./code.cg2js/cg-property.fasl")
(#x725293e5feac2577f0209941b6860d5b . "./code.cg2js/cg-menu.fasl")
(#xb1e88dce9f6008bce25f2cbba15aa971 . "./code.cg2js/cg-clipboard.fasl")
(#xc57741ca7eceb3a9813d8597b75bb959 . "./code.cg2js/cg-toggling-widget.fasl")
(#x38ab4365c4869908023c41ab9c28ed49 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xe293d72a4f2c10e11609e8b3c4320143 . "./code.cg2js/cg-template-string.fasl")
(#xedb1b80eb143652d0e78150946b7ffb1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd0152f88e06a29dead25138518cdd06a . "./code.cg2js/cg-os-window.fasl")
(#xe9e66518d2d0195104a1fe5490de9720 . "./code.cg2js/cg-grid-widget.fasl")
(#x4c3b708a65a96bd54c6c39fc54173ad5 . "./code.cg2js/cg-picture-widget.fasl")
(#x10e717ca2c3dbccdb7688f0d6ee7f6fe . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x4e5d2d4028b63725f511910704775477 . "./code.cg2js/cg-static-text.fasl")
(#xa77989d8c4dd0703f0ca00d19066b131 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xc1ff81232fb8180de841ff1ccc60c05d . "./code.cg2js/cg-text-widget.fasl")
(#x2a43d85761665cd07772c3c5dbce1195 . "./code.cg2js/cg-calendar.fasl")
(#xbde84a764c4befed6062f2d6ee59265c . "./code.cg2js/cg-check-box.fasl")
(#x36d8e5ad0da014b5b49eddd2402f4428 . "./code.cg2js/cg-acache.fasl")
(#x2f22d024681876c5cceba65043d76750 . "./code.cg2js/cg-wrap-string.fasl")
(#x687c92d7173db3e63a608080637969ae . "./code.cg2js/cg-bitmap-stream.fasl")
(#x5e633717f8129b6ed6667b27f6973cb6 . "./code.cg2js/cg-font-dialog.fasl")
(#x457a7038fe2f1a36938579e72295e925 . "./code.cg2js/cg-outline.fasl")
(#xc1a643594a6315f858924f1310f20ebc . "./code.cg2js/cg-shortcut-menu.fasl")
(#x53ce2a3a927c972b42de9d157a996a70 . "./code.cg2js/cg-status-bar.fasl")
(#x732fa19a877c29aaf38f18ca3b75d710 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x5268fbe01c5e59a0b64b6018cd1061d8 . "./code.cg2js/cg-hotspot.fasl")
(#xfe562b8ac3d02a07a0342b40700c1383 . "./code.cg2js/cg-scaling-stream.fasl")
(#xc846497fa65abdf0dc0bdca975bf3665 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x644e9de6f0008992c5933ff6b16dd0c0 . "./code.cg2js/cg-file-dialog.fasl")
(#x7d157dcaa9d7f0d1a230f20e8339075e . "./code.cg2js/cg-color-dialog.fasl")
(#xa4e8977db25e073cda5c3ec3fcde297e . "./code.cg2js/cg-get-pixmap.fasl")
(#x01f14c148f448b43a40e9639dc86fc88 . "./code.cg2js/cg-object-editor.fasl")
(#x2f7eb4401e3217b63b79fe06446fa32c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x658e4f64a8aa00e5e88c8e3241b207b4 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x3691f8fa4f555974f5f777b01f612959 . "./code.cg2js/cg-pixmap.fasl")
(#x11ab6fdc1cc6580ae7acac32adbd04e6 . "./code.cg2js/cg-intersect.fasl")
(#x97145d54335cc431fe8a7af65dbb4831 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xadf6a515dee25f810061625426d89505 . "./code.cg2js/cg-icon.fasl")
(#xaa677ffea542ed25b89ae05a641644d2 . "./code.cg2js/cg-yes-no-string.fasl")
(#x4122a43e3d98caf71d90615e67fcb25d . "./code.cg2js/cg-text-or-combo.fasl")
(#xf2ee38d21e3e60fb770cc5eed394b587 . "./code.cg2js/cg.fasl")
(#xe91796ecc6df4c1af75f1903d335f10d . "./code.cg2js/cg-dialog-item.fasl")
(#xfa6f4cfd803d902131ba05cd8a825649 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x696fb36913c7380c5d54a66d198d94b5 . "./code.cg2js/cg-caret.fasl")
(#x2c71ccd3910fff49ae80ce94f3242660 . "./code.cg2js/cg-editable-text.fasl")
(#x4b6b7c068200833d9c7db4ce24b1632c . "./code.cg2js/cg-choose-printer.fasl")
(#x00f8e45a00a5ec801c7f41568464e496 . "./code.cg2js/cg-yes-no-list.fasl")
(#xcd5e2740865c53469aaf83d3dc7496af . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe441e04bbb6ef8911b85e6c64424a025 . "./code.cg2js/cg-tooltip.fasl")
(#xe6e19b6d22969dff4cc9fc48dfa837aa . "./code.cg2js/cg-class-slot-grid.fasl")
(#xdadeb055015c89f91c51ab469823b499 . "./code.cg2js/cg-combo-box.fasl")
(#xe8063a9fadce7c9de7283fb4d137b4da . "./code.cg2js/cg-curve.fasl")
(#x1560e0aacdb231c3dd997a76c70e3a14 . "./code.cg2js/cg-web-browser.fasl")
(#xedb4a13b2791dc3f41d8fd5d938d0e73 . "./code.cg2js/cg-printing.fasl")
(#x1ead6b95f16e593c4ee63658ff815c11 . "./code.cg2js/cg-lisp-widget.fasl")
(#xd2b014120f184844e9d3573fd6885ee7 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xd23752420fe556d0bcfad2bf6f6c935a . "./code.cg2js/cg-palette.fasl")
(#x0f65d10a8c6ba545d1fa1e67fae91c5b . "./code.cg2js/cg-os-widget.fasl")
(#xb56e598dd3e5c17fd10bf92ff3c60a66 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x7e3081b08e3f14d141f5f56578884bbb . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe9d830d957a5f3ea608ddb43dc2b041c . "./code.cg2js/cg-button.fasl")
(#xc270cef49842884943c45d30ef3598ff . "./code.cg2js/cg-edit-in-place.fasl")
(#x01a3f69ce9db2d8fbd9aae701d4ed462 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x3a38517be16c2b94a6d2ad1cce2576df . "./code.cg2js/cg-comtab.fasl")
(#xb2eef67d8e3fe65070874c86a1e13065 . "./code.cg2js/cg-string-dialog.fasl")
(#x7e8d3fff654453bdcf29c9edc75d1dd9 . "./code.cg2js/cg-header-control.fasl")
(#xd4f6da317cb4c713f029a9ee7a7e08a3 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xcc1420e7b3581690f82529d2d76a77cf . "./code.cg2js/cg-up-down-control.fasl")
(#xc88c029309fe7b5ce7cedc2e3b649cc5 . "./code.cg2js/cg-layout.fasl")
(#x0895f5f65f5b69e892531eacc16ca507 . "./code.cg2js/cg-chart-widget.fasl")
(#x6d37ebc2501d79368b699329454085a2 . "./code.cg2js/cg-lisp-text.fasl")
(#xc8abe4221e7f94076101b39fc4b9d6b6 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x148091d9d0f3eb4d6c915dfff0cc3b90 . "./code.cg2js/cg-progress-indicator.fasl")
(#x55de667a78a67354b6cf4990bdf75103 . "./code.idejs/ide-inspector.fasl")
(#x6a4e09919db2f083469af53ecaf85f33 . "./code.idejs/ide-grapher.fasl")
(#x316d48a08158b87b55d632cafef80f4b . "./code.idejs/ide-base.fasl")
(#x17044dbb088498439b9c14e238303bdc . "./code.idejs/ide-keyshort.fasl")
(#xf11f9dbe5cf16e91b7c3b1029511b32c . "./code.idejs/ide-dev.fasl")
(#x393adabe553d6fa5d59dac108faa1035 . "./code.idejs/ide-class-browser.fasl")
(#xa693b7df0475de3e4af24162be2e7902 . "./code.idejs/ide-package-list.fasl")
(#x20d861fc71a48ac494f343313adb73ce . "./code.idejs/ide-intro.fasl")
(#xed86f07389d713ea4fd1db7e65e97218 . "./code.idejs/ide-class-grapher.fasl")
(#x91126925974c128873b61377e305d2de . "./code.idejs/ide-window-list.fasl")
(#x827254c13848524f05a8539ecea7245c . "./code.idejs/ide-find-definitions.fasl")
(#xffa8db57bf4deb5203145c6384303b3a . "./code.idejs/ide-trace.fasl")
(#xa8578bcc441194c486d9026244266779 . "./code.idejs/ide-editor.fasl")
(#x2dc13fbec287657d441792fd7135a1f1 . "./code.idejs/ide-patch-dialog.fasl")
(#x70848050f15d4fe7df1cb2e36b7935b4 . "./code.idejs/ide-project.fasl")
(#x19f584745ee5a3c4aa03ee9e7ecfb107 . "./code.idejs/ide.fasl")
(#xc185bc7b4abf27a725875a03a0f91fe4 . "./code.idejs/ide-apropos.fasl")
(#xca19e0cd99f636b916f440a410f7357a . "./code.idejs/ide-acache.fasl")
(#x3c2ec3b615ac2310925732e1c9f410e4 . "./code.idejs/ide-clipboard.fasl")
(#x4106e34933d91b0770ff02c78a536804 . "./code.idejs/ide-process-browser.fasl")
(#x0bd94b8d8808cba3c46da8f5de9b37f6 . "./code.idejs/ide-find-in-files.fasl")
(#x1fffbbee96b29f49b3eb736a667226dc . "./code.idejs/ide-form.fasl")
(#x157976cb2ef847571f76d0a8dffaeae4 . "./code.idejs/ide-navigator.fasl")
(#x785b009207cd19fe54c93fc535e484ec . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
